// These button classes are based on the Button component from: 
// JasmineTunes 2
// Button component, v1.0
// Coded by: M Amroabadi


/* Definerer en vanlig knapp */
function Button(imageobj, normalPath, activePath){
	//-----------------------------------------------------------
	// Private Properties
	//-----------------------------------------------------------
	// Holder styr p bilde objektet og bildene som skal vises p knappen
	this.Image = imageobj;
	this.NormalSrc = normalPath;
	this.ActiveSrc = activePath;
	// Interne variabler. Holder styr p statusen p knappene
	this.MouseOver = false;
	this.MouseDown = false;
	this.Enabled = true;
	// Default funksjon for eventer
	this.DefaultHandler = function(){};
	
	//-----------------------------------------------------------
	// Public Methods
	//-----------------------------------------------------------
	// Setter normalt bilde. Bilde som vises nr knappen ikke rres
	this.SetNormalImage = function(path){
		this.NormalSrc = path;
		this.Image.src = path;
		}
	// Setter aktivt bilde. Bilde som vises nr knappen trykkes ned
	this.SetActiveImage = function(path){
		this.ActiveSrc = path;
		}
	// Aktiverer knappen
	this.Enable = function(){
		this.Image.opacity = 255;
		this.Enabled = true;
		}
	// Deaktiverer knappen
	this.Disable = function(){
		this.Image.opacity = 150;
		this.Enabled = false;
		// Setter andre interne variabler tilbake til normalt
		this.Image.src = this.NormalSrc;
		this.MouseOver = false;
		this.MouseDown = false;
		}
	
	//-----------------------------------------------------------
	// Public Event Handlers
	//-----------------------------------------------------------
	// Benyttes til  sette funksjonalitet p knappene
	this.onClickRelease = this.DefaultHandler;		// Nr museknappen slippes opp
	this.onPress = this.DefaultHandler;				// Nr museknappen er nede
	this.onExit = this.DefaultHandler;				// Nr musepekeren forsvinner ut av knappen
	
	// Constructor: Image Event Handler Hookup. Setter funksjonalitet p knappens originale eventer og kaller p knappens public event handlers
	this.Image.onMouseDown = function(){
		if(this.Enabled){
			this.MouseDown = true;
			if(this.MouseOver){
				this.Image.src = this.ActiveSrc;
				this.onPress();
				}
			}
		}.bind(this);
	this.Image.onMouseEnter = function(){
		if(this.Enabled){
			this.MouseOver = true;
			if(this.MouseDown){
				this.Image.src = this.ActiveSrc;
				this.onPress();
				}
			}
		}.bind(this);
	this.Image.onMouseExit = function(){
		if(this.Enabled){
			this.MouseOver = false;
			if(this.MouseDown){
				this.Image.src = this.NormalSrc;
				this.onExit();
				}
			}
		}.bind(this);
	this.Image.onMouseUp = function(){
		if(this.Enabled){
			this.MouseDown = false;
			this.Image.src = this.NormalSrc;
			if(this.MouseOver){
				this.onClickRelease();
				}
			}
		}.bind(this);
	}

/* Definerer en knapp som har funksjonalitet nr musepekeren holdes inne over lengre perioder */
function PressButton(imageobj, normalPath, activePath){
	//-----------------------------------------------------------
	// Private Properties
	//-----------------------------------------------------------
	// Interne variabler. Holder styr p bilde objektet og bildene som skal vises p knappen
	this.Image = imageobj;
	this.NormalSrc = normalPath;
	this.ActiveSrc = activePath;
	// Interne variabler. Holder styr p statusen p knappene
	this.Timestamp = 0;
	this.PressTimeout = 300;
	this.MouseOver = false;
	this.MouseDown = false;
	this.Enabled = true;
	// Default funksjon for eventer
	this.DefaultHandler = function(){};
	
	//-----------------------------------------------------------
	// Public Methods
	//-----------------------------------------------------------
	// Setter tiden som definerer hva som er et klikk og hva som er et "press and hold" klikk
	this.SetPressTimeout = function(timeout){
		this.PressTimeout = timeout;
		}
	// Aktiverer knappen
	this.Enable = function(){
		this.Image.opacity = 255;
		this.Enabled = true;
		}
	// Deaktiverer knappen
	this.Disable = function(){
		this.Image.opacity = 128;
		this.Enabled = false;
		// Setter andre interne variabler tilbake til normalt
		this.Image.src = this.NormalSrc;
		this.MouseOver = false;
		this.MouseDown = false;
		this.Timestamp = 0;
		}
	
	//-----------------------------------------------------------
	// Public Event Handlers
	//-----------------------------------------------------------
	// Benyttes til  sette funksjonalitet p knappene
	//this.onClick = this.DefaultHandler;
	this.onClickRelease = this.DefaultHandler;		// Nr museknappen slippes opp og tiden er mindre enn Timestamp (ett klikk)
	this.onPress = this.DefaultHandler;				// Nr museknappen er nede
	this.onPressRelease = this.DefaultHandler;		// Nr museknappen slippes opp og tiden er mer enn Timestamp (ett "press and hold" klikk)
	this.onExit = this.DefaultHandler;				// Nr pekeren forsvinner ut av knappen
		
	// Constructor: Image Event Handler Hookup. Setter funksjonalitet p knappens originale eventer og kaller p knappens public event handlers
	this.Image.onMouseDown = function(){
		if(this.Enabled){
			this.MouseDown = true;
			this.Timestamp = fxGetTimestamp();
			if(this.MouseOver){
				this.Image.src = this.ActiveSrc;
				this.onPress();
				}
			}
		}.bind(this);
	this.Image.onMouseEnter = function(){
		if(this.Enabled){
			this.MouseOver = true;
			if(this.MouseDown){
				this.Image.src = this.ActiveSrc;
				this.onPress();
				}
			}
		}.bind(this);
	this.Image.onMouseExit = function(){
		if(this.Enabled){
			this.MouseOver = false;
			if(this.MouseDown){
				this.Image.src = this.NormalSrc;
				this.onExit();
				}
			}
		}.bind(this);
	this.Image.onMouseUp = function(){
		if(this.Enabled){
			this.MouseDown = false;
			this.Image.src = this.NormalSrc;
			if(this.MouseOver){
				if((fxGetTimestamp() - this.Timestamp) < this.PressTimeout){
					this.onClickRelease();
					}
				else{
					 this.onPressRelease();
					}
				}
			}
		}.bind(this);
	}