var Player = {
	Running: isApplicationRunning("itunes.exe")
	, Version: 0		
	, PlaylistName: "Album Browser Widget Playlist"		
	, PlaylistCache: null			
	, get State(){
		var value = "unknown";
		try{
			if(this.Running){
				switch(iTunesObj.PlayerState){
					case 0:
						value = (iTunesObj.CurrentTrack == null)?"stopped":"paused";
						break;
					case 1:
						value = "playing";
						break;
					case 2:
						value = "fast forwarding";
						break;
					case 3:
						value = "rewinding";
						break;	
					}
				}
			}
		catch(ex){
			log("error: " + ex);
			}
		return value;
		}
	, get Position(){
		return (this.Running)?iTunesObj.PlayerPosition:0;
		}
	, get Volume(){
		return (this.Running)?iTunesObj.SoundVolume:0;
		}
	, set Volume(newvolume){
		if(this.Running){
			iTunesObj.SoundVolume = Math.min(Math.max(0, newvolume),100);
			}
		}
	, get Rating(){
		if(this.Running){
			if(iTunesObj.CurrentTrack != null){
				return iTunesObj.CurrentTrack.Rating;
				}
			}
		}
	, set Rating(rating){
		if(this.Running){
			if(iTunesObj.CurrentTrack != null){
				iTunesObj.CurrentTrack.Rating = rating;
				}
			}
		}
	, Track: {
		Album: ""
		, Artist: ""
		, Name: ""
		, Rating: 0
		, Kind: "unknown"
		, Duration: 0
		, Time: "0:00"
		, StreamTitle: ""
		, StreamURL: ""
		, IsFileTrack: false
		, IsVideo: false
		, IsStream: false
		, IsPodcast: false
		, IsCompilation: false
		, TrackId: ""
		}
	, GetTrack: function(){
		return this.Track;
		}
	, RefreshTrack: function(){
		if(this.IsPlaying()){
			this.ChangeTrack(iTunesObj.CurrentTrack);
			}
		}
	, ChangeTrack: function(track){
		this.Track.Album = track.Album;
		this.Track.Artist = track.Artist;
		this.Track.Name = track.Name;
		this.Track.Rating = track.Rating;
		this.Track.TrackId = track.TrackDatabaseID;
		this.Track.Duration = track.Duration;
		this.Track.Time = track.Time
		var kind;
		switch(track.Kind){
			case 0:
				kind = "unknown";
				break;
			case 1:
				kind = "file track";
				break;
			case 2:
				kind = "cd track";
				break;
			case 3:
				kind = "url track";
				break;
			case 4:
				kind = "device track";
				break;
			case 5:
				kind = "shared library";
				break;
			}
		this.Track.Kind = kind;
		this.Track.IsFileTrack = (kind == "file track")?true:false;
		this.Track.IsPodcast = track.Podcast;
		this.Track.IsCompilation = track.Compilation;
		if(kind == "url track"){
			this.Track.IsStream = true;
			this.Track.StreamTitle = iTunesObj.CurrentStreamTitle;
			this.Track.StreamURL = iTunesObj.CurrentStreamURL;
			}
		else{
			this.Track.IsStream = false;
			this.Track.StreamTitle = "";
			this.Track.StreamURL = "";
			}
		if(kind == "file track" || kind == "cd track"){
			this.Track.IsVideo = (iTunes.Version >= "7")?(track.VideoKind >= 1):(track.KindAsString.indexOf("MPEG-4 video") > -1);
			}
		else{
			this.Track.IsVideo = false;
			}
		}
	, ClearTrack: function(){	
		this.Track.Album = "";
		this.Track.Artist = "";
		this.Track.Name = "";
		this.Track.Rating = "";
		this.Track.Kind = "unknown";
		this.Track.Duration = 0;
		this.Track.Time = "0:00";
		this.Track.StreamTitle = "";
		this.Track.StreamURL = "";
		this.Track.IsFileTrack = false;
		this.Track.IsVideo = false;
		this.Track.IsStream = false;
		this.Track.IsPodcast = false;
		this.Track.IsCompilation = false;
		this.Track.TrackId = "";
		}
	, GetCurrentPlaylist: function(){
		return iTunesObj.CurrentPlaylist;
		}
	, Playlist: {
		get Shuffle(){
			var playlist = iTunesObj.CurrentPlaylist;
			return (playlist != null)?playlist.Shuffle:false;
			}
		, set Shuffle(value){
			var playlist = iTunesObj.CurrentPlaylist;
			if(playlist != null){
				playlist.Shuffle = value;
				}
			}
		, get Repeat(){
			var value;
			var playlist = iTunesObj.CurrentPlaylist;
			if(playlist != null){
				switch(iTunesObj.CurrentPlaylist.SongRepeat){
					case 0:
						value = "off";
						break;
					case 1:
						value = "one";
						break;
					case 2:
						value = "all";
						break;
					}
				}
			else{
				value = "off";
				}
			return value;
			}
		, set Repeat(input){
			var value;
			var playlist = iTunesObj.CurrentPlaylist;
			if(playlist != null){
				switch(input){
					case "off":
						value = 0;
						break;
					case "one":
						value = 1;
						break;
					case "all":
						value = 2;
						break;
					}
				iTunesObj.CurrentPlaylist.SongRepeat = value;	
				}
			}
		}
	, BackTrack: function(){
		if(this.Running){
			iTunesObj.BackTrack();
			}
		}
	, FastForward: function(){
		if(this.Running){
			iTunesObj.FastForward();
			}
		}
	, NextTrack: function(){
		if(this.Running){
			iTunesObj.NextTrack();
			}
		}
	, Pause: function(){
		if(this.Running){
			iTunesObj.Pause();
			}
		}
	, Play: function(){
		if(this.Running){
			iTunesObj.Play();
			}
		}
	, PlayPause: function(){
		if(this.Running){
			iTunesObj.PlayPause();
			}
		}
	, Resume: function(){
		if(this.Running){
			iTunesObj.Resume();
			}
		}
	, Rewind: function(){
		if(this.Running){
			iTunesObj.Rewind();
			}
		}
	, Stop: function(){
		if(this.Running){
			iTunesObj.Stop();
			}
		}
	, GetCurrentTrack: function(){
		if(this.Running){
			if(iTunesObj.CurrentTrack != null){
				return iTunesObj.CurrentTrack;
				}
			}
		return null;
		}
	, IsPlaying: function(){
		return (this.State == "playing");
		}
	, IsPaused: function(){
		return (this.State == "paused");
		}
	, IsPlayingOrPaused: function(){
		return ( this.IsPlaying() || this.IsPaused() );
		}
	, CheckState: function(){
		try{
			this.Running = isApplicationRunning("itunes.exe");
			switch (Widget.CurrentStatus){
				case "started":
					if(!this.Running){
						this.DisconnectEvents();
						Widget.CurrentStatus = "wait";
						this.SetStateOnControls();
						}
					else{
						var state = this.State;
						switch(state){
							case "playing":
								var track = iTunesObj.CurrentTrack;
								if(track != null){
									if(track.TrackDatabaseID != Player.Track.TrackId){
										iTunes_OnPlayerPlayEvent(iTunesObj.CurrentTrack);								
										}
									}
								break;
							case "stopped":
							case "paused":
								if(this.PullState.CurrentState != state){
									iTunes_OnPlayerStopEvent();
									}
								break;
							default:
								break;	
							}
						this.PullState.CurrentState = state;
						}
					break;
				case "stopping":
					if(!this.Running){
						Widget.CurrentStatus = "wait";
						this.SetStateOnControls();
						}
					break;
				case "wait":
					if(this.Running){
						this.ConnectEvents();
						Widget.CurrentStatus = (iTunesObj != null)? "started":"wait";
						if(!UI.Iconized){
							if( this.IsRunning() ){
								if(UI.TrackDisplay){
									if(this.IsPlaying() && !TrackDisplay.Started){
										TrackDisplay.Refresh(false);
										}
									}
								this.SetStateOnControls();
								}
							}
						}
					break;
				case "restart":
					this.RunApplication((preferences.StartiTunesOnLoad.value == 1));
					if(!UI.Iconized){
						if( this.IsRunning() ){
							if(UI.TrackDisplay){
								if(this.IsPlaying() && !TrackDisplay.Started){
									TrackDisplay.Refresh(false);
									}
								}
							this.SetStateOnControls();
							}
						}
					break;
				}
			}
		catch(ex){
			log("error: " + ex);
			}
		}
	, PollState: function(){
		try{
			this.Running = isApplicationRunning("itunes.exe");
			switch (Widget.CurrentStatus){
				case "started":
					if(!this.Running){
						this.DisconnectEvents();
						Widget.CurrentStatus = "wait";
						this.SetStateOnControls();
						fxCloseCurrentState();
						}
					else{
						var state = this.State;
						switch(state){
							case "playing":
								var track = iTunesObj.CurrentTrack;
								if(track != null){
									var trackid = "";
									if(track.Kind != 3){
										trackid = track.TrackDatabaseID;
										}
									else{
										trackid = track.Name + track.Album + track.Artist + iTunesObj.CurrentStreamTitle;
										}
									if(this.PullState.CurrentTrackId != trackid){
										iTunes_OnPlayerPlayEvent(track);
										this.PullState.CurrentTrackId = trackid
										}
									else{
										if(track.Rating != Player.Track.Rating){
											iTunes_OnPlayerPlayingTrackChangedEvent(track);
											}
										}
									}
								break;
							case "stopped":
							case "paused":
								if(this.PullState.CurrentState != state){
									iTunes_OnPlayerStopEvent();
									this.PullState.CurrentTrackId = "";
									}
								break;
							case "fast forwarding":
							case "rewinding":
								break;
							default:
								StateTimer.interval = 5;
								Widget.CurrentStatus = "stopping";
								break;	
							}
						this.PullState.CurrentState = state;
						}
					break;
				case "stopping":
					if(!this.Running){
						Widget.CurrentStatus = "wait";
						this.SetStateOnControls();
						StateTimer.interval = 10;
						fxCloseCurrentState();
						this.PullState.RetryCount = 0;
						}
					else{
						this.PullState.RetryCount += 1;
						if(this.PullState.RetryCount >= 4){
							StateTimer.interval = 3;
							Widget.CurrentStatus = "started";
							this.PullState.RetryCount = 0;
							}
						}
					break;
				case "wait":
					if(this.Running){
						this.ConnectEvents();
						Widget.CurrentStatus = (iTunesObj != null)? "started":"wait";
						StateTimer.interval = 3;
						if(!UI.Iconized){
							if( this.IsRunning() ){
								if(UI.TrackDisplay){
									if(this.IsPlaying() && !TrackDisplay.Started){
										TrackDisplay.Refresh(false);
										}
									}
								this.SetStateOnControls();
								}
							}
						}
					break;
				case "restart":
					this.RunApplication((preferences.StartiTunesOnLoad.value == 1));
					if(!UI.Iconized){
						if( this.IsRunning() ){
							StateTimer.interval = 3;
							if(UI.TrackDisplay){
								if(this.IsPlaying() && !TrackDisplay.Started){
									TrackDisplay.Refresh(false);
									}
								}
							this.SetStateOnControls();
							}
						}
					break;
				}
			}
		catch(ex){
			log("error: " + ex);
			}
		}
	, PullState:{
		CurrentTrackId: ""
		, CurrentState: ""
		, RetryCount: 0
		}
	, IsRunning: function(){
		return Widget.CurrentStatus == "started" && isApplicationRunning("itunes.exe");
		}
	, ConnectEvents: function(){
		try{
			iTunesObj = COM.createObject("iTunes.Application");	
			if(preferences.iTunesStatusMethod.value == 1){
				COM.connectObject(iTunesObj, "iTunes_");		
				}
			}
		catch(ex){
			log("__iTunes not installed or error occured:" + ex);
			iTunesObj = null;
			}
		}
	, DisconnectEvents: function(){
		if(iTunesObj != null && preferences.iTunesStatusMethod.value == 1){
			COM.disconnectObject(iTunesObj);
			}
		iTunesObj = null;
		}
	, RunApplication: function(setState){
		this.ConnectEvents();
		if(iTunesObj == null){
			if(preferences.StartiTunesOnLoad.value == 1){
				Widget.CurrentStatus = "restart";
				StateTimer.ticking = true;
				}
			else{
				Widget.CurrentStatus = "wait";
				}
			}
		else{
			Widget.CurrentStatus = "started";
			this.playerStatus;
			if(setState){
				this.SetState(preferences.iTunesStateOnLoad.value);
				}
			UI.Focus();		
			if(Widget.Loaded){
				this.SetStateOnControls();
				}
			this.Version = iTunesObj.Version;
			}
		}
	, Quit: function(){
		Widget.CurrentStatus = "stopping";
		iTunesObj.Quit();
		iTunesObj = null;
		fxCloseCurrentState();
		this.SetStateOnControls();
		}
	, Focus: function(){
		if(iTunesObj.BrowserWindow.MiniPlayer){
			iTunesObj.BrowserWindow.MiniPlayer = false;
			}
		else{
			iTunesObj.BrowserWindow.Visible = true;
			iTunesObj.BrowserWindow.Minimized = false;
			}
		}
	, Toggle: function(){
		if(this.IsRunning()){
			if(iTunesObj.BrowserWindow.Minimized){
				iTunesObj.BrowserWindow.Minimized = false;
				}
			else{
				if(iTunesObj.BrowserWindow.MiniPlayer){
					iTunesObj.BrowserWindow.MiniPlayer = false;
					}
				else{
					iTunesObj.BrowserWindow.Minimized = true;
					}
				}
			}
		}
	, SetState: function(state){
		switch (state){
			case "1": 
				this.Focus();
				break;
			case "2": 
				iTunesObj.BrowserWindow.Minimized = true;
				break;
			case "3":
				iTunesObj.BrowserWindow.MiniPlayer = true;
				break;
			default:
				this.Focus();
				break;
			}
		}
	, ShowMusicStore: function(){
		iTunesObj.GotoMusicStoreHomePage();
		}
	, ShowMusicStoreFeature: function(feature){
		var url = "";
		switch(feature){
			case "JustForYou":
				url = "itms://phobos.apple.com/WebObjects/MZPersonalizer.woa/wa/seeAllRecommendations?js=0";
				break;
			case "MovieTrailers": 
				url = "itms://phobos.apple.com/WebObjects/MZStore.woa/wa/inboundRedirect?omni-ch=Store+Front&omni-pg=Movie+Trailers&url=http%3A%2F%2Fwww.apple.com%2Fmoviesxml%2Fh%2Findex.xml";
				break;
			case "TopSongs": 	
				url = "itms://phobos.apple.com/WebObjects/MZStore.woa/wa/viewTop?popId=1";
				break;
			case "TopAlbums": 
				url = "itms://phobos.apple.com/WebObjects/MZStore.woa/wa/viewTop?popId=11";
				break;
			}
		if(url != ""){ this.OpenMusicStoreURL(url); }
		}
	, ShowAlbum: function(artist, title){
		var url = "itms://phobos.apple.com/WebObjects/MZSearch.woa/wa/advancedSearchResults?";
		if(artist == Const.CompilationAlbum){
			url += "albumTerm="+ urlString(title);
			}
		else{
			url += "artistTerm="+ urlString(artist) +"&albumTerm="+ urlString(title);
			}
		this.OpenMusicStoreURL(url);
		}
	, ShowArtist: function(artist){
		var url = "itms://phobos.apple.com/WebObjects/MZSearch.woa/wa/advancedSearchResults?artistTerm="+ urlString(artist);
		this.OpenMusicStoreURL(url);
		}
	, OpenMusicStoreURL: function(url){
		try{
			iTunesObj.OpenURL(url);
			}
		catch (ex){
			log("__Error opening iTunes Store Url: " + ex);
			}
		}
	, SaveArtworkToFile: function(Artwork, folder, name){
		var artworkPath = Const.NoArtworkImage;
		if(Artwork.Count > 0){
			var fileExtention = fxGetArtworkFormat(Artwork.Item(1).Format);
			if(fileExtention != null){
				try{
					artworkPath = convertPathToPlatform( folder +"/"+ name + fileExtention, true);
					Artwork.Item(1).SaveArtworkToFile( artworkPath );
					}
				catch(ex){
					log("__Error saving artwork image: " + ex)
					artworkPath = Const.NoArtworkImage;
					}
				}
			}
		return artworkPath;
		}
	, DeleteAlbumArtwork: function(key){
		try{
			if(UI.Busy){ return; }
			var currentalbum = Albums.ItemByKey(key);
			var title = currentalbum.Title;
			var artist = currentalbum.Artist;
			var deletedCount = 0;
			if(currentalbum.HasArtwork){
				switch(currentalbum.ArtworkSource){
					case 1: 
						if(alert(AlertMsg.DeleteArtwork + "\n\nAlbum: " + title + "\nArtist:  " + artist, "Delete Artwork", "Cancel") == 2){
							currentalbum = null;
							return;
							}
						var albumTracks = this.Search(title);
						if(albumTracks != null){
							UI.ShowResultInfo("Deleting Artwork from Album...");
							UI.Busy = true;			
							if(UI.TrackDisplay){
								TrackDisplay.Stop();
								}
							StateTimer.ticking = false;			
							ArtworkTitlefield.scrolling = "off";	
							var albumCount = albumTracks.Count;
							for(var k = 1; k <= albumCount; k++){
								var track = albumTracks.Item(k);
								if(trim(track.Album) == title){
									if(!Library.UseCompilation && track.Compilation){
										track = null;
										continue;
										}
									if( (!track.Compilation)?(trim(track.Artist) == artist):true ){
										if(track.Artwork.Count > 0){
											track.Artwork.Item(1).Delete();
											deletedCount += 1;
											sleep(100);
											}
										}
									}
								track = null;
								}
							albumTracks = null;
							}
						else{
							UI.ShowResultInfo("Delete Artwork: No Album found");
							}
						break;
					default:
						UI.ShowResultInfo("No Artwork deleted");
						currentalbum = null;
						return;
						break;
					} // end switch
				if(deletedCount > 0){
					var tmpPath = currentalbum.GetArtworkPath();
					if(filesystem.itemExists(tmpPath)){
						filesystem.moveToRecycleBin(tmpPath);
						}
					tmpPath = currentalbum.GetThumbnailPath();
					if(filesystem.itemExists(tmpPath)){
						filesystem.moveToRecycleBin(tmpPath);
						}
					if(currentalbum.HasReflection){
						tmpPath = currentalbum.GetReflectionPath();
						if(filesystem.itemExists(tmpPath)){
							filesystem.moveToRecycleBin(tmpPath);
							}
						}
					currentalbum.ArtworkName = "";
					currentalbum.ArtworkSource = 0;
					currentalbum.HasArtwork = false;
					currentalbum.HasReflection = false;
					Albums.Update(currentalbum.Id, currentalbum);
					Browser.Reset();
					Browser.ResetAlbums();	
					Browser.ShowAlbumByKey(key);	
					UI.ShowArtworkWindow(key);
					if(Floater.Exists(currentalbum.Id)){
						Floater.Update(currentalbum.Id);
						}
					}
				else{
					UI.ShowResultInfo("No Artwork deleted");
					}
				}
			currentalbum = null;
			}
		catch(ex){
			log("__Error deleting artwork from album:" + ex);
			}
		UI.Busy = false;	
		StateTimer.ticking = true;
		ArtworkTitlefield.scrolling = "autoLeft";	
		fxCheckTrackDisplay();	
		}
	, AddAlbumArtwork: function(key, method){
		try{
			if(UI.Busy){ return; }
			var currentalbum = Albums.ItemByKey(key);	
			var title = currentalbum.Title;
			var artist = currentalbum.Artist;
			var isCompilationAlbum = false;
			if(currentalbum.ArtworkSource != 1){
				var filePath = "";
				switch(method){
					case "manual":
						var formfields = Array();
						formfields[0] = new FormField();
						formfields[0].name = "imagePath";
						formfields[0].type = "selector";
						formfields[0].style = "open";
						formfields[0].kind = "files";
						formfields[0].title = "Image Path:";
						formfields[0].description = "Browse to the artwork image you want to add to the album:" + "\n\nAlbum: " + title + "\nArtist:  " + artist;
						var formResults = form(formfields, "Add Artwork to Album", "Add Artwork");
						if(formResults != null) {
							filePath = formResults.toString();
							if(!filesystem.itemExists(filePath)){ 
								UI.ShowResultInfo("No Artwork added");
								currentalbum = null;
								return;
								}
							} 
						else{
							return;	
							}
						break;
					case "dragdrop":
						filePath = system.event.data[1];
						if(alert(AlertMsg.AddArtwork + "\n\nAlbum: " + title + "\nArtist:  " + artist, "Add Artwork", "Cancel") == 2){
							currentalbum = null;
							return;
							}
						break;
					default:
						UI.ShowResultInfo("No Artwork added");
						currentalbum = null;
						return;
						break;
					}		
				var fileFormat = fxGetFileFormat(filePath);
				if( fxIsValidImageFormat(fileFormat) ){
					filePath = convertPathToPlatform(filePath, true);
					var albumTracks = this.Search(title);
					if(albumTracks != null){
						UI.ShowResultInfo("Adding Artwork to Album...");
						UI.Busy = true;
						if(UI.TrackDisplay){
							TrackDisplay.Stop();
							}
						StateTimer.ticking = false;	
						ArtworkTitlefield.scrolling = "off";	
						var albumCount = albumTracks.Count;
						var addedCount = 0;
						for(var k = 1; k <= albumCount; k++){
							var track = albumTracks.Item(k);
							if(trim(track.Album) == title){
								if(!Library.UseCompilation && track.Compilation){
									track = null;
									continue;
									}
								if( (!track.Compilation)?(trim(track.Artist) == artist):true ){
									if(track.Artwork.Count == 0){
										var obj = track.AddArtworkFromFile(filePath);
										obj = null;
										addedCount += 1;
										sleep(100);
										}
									}
								if(track.Compilation && !isCompilationAlbum){
									isCompilationAlbum = true;
									}
								}
							track = null;
							}
						albumTracks = null;
						if(addedCount > 0){
							if(isCompilationAlbum){
								artist = Const.CompilationAlbum;
								}
							var newFileName = cleanString(artist +"-"+ title) + fileFormat;
							var newArtworkPath = convertPathToPlatform(Const.ArtworkFolder + "/" + newFileName, true);
							var newThumbnailPath = convertPathToPlatform(Const.ThumbnailFolder + "/" + newFileName, true);
							if(filesystem.itemExists(newArtworkPath)){
								filesystem.moveToRecycleBin(newArtworkPath);
								}
							if(filesystem.itemExists(newThumbnailPath)){
								filesystem.moveToRecycleBin(newThumbnailPath);
								}
							filesystem.copy( filePath, newArtworkPath );
							fxRunThumbnailCmd( "\"" + newArtworkPath + "\"",  "\"" + newThumbnailPath + "\"");
							if(UI.AlbumReflection){
								var newReflectionPath = convertPathToPlatform(Const.ReflectionFolder + "/" + newFileName, true);
								if(filesystem.itemExists(newReflectionPath)){
									filesystem.moveToRecycleBin(newReflectionPath);
									}
								fxRunReflectionCmd( "\"" + newThumbnailPath + "\"",  "\"" + newReflectionPath + "\"");
								currentalbum.HasReflection = true;
								}
							currentalbum.ArtworkName = newFileName;
							currentalbum.ArtworkSource = 1;
							currentalbum.HasArtwork = true;
							Albums.Update(currentalbum.Id, currentalbum);
							Browser.Reset();
							Browser.ResetAlbums();	
							UI.ShowArtworkWindow(key);
							if(UI.Minimized){
								UI.Minimize();
								}
							Browser.ShowAlbumByKey(key);
							if(Floater.Exists(currentalbum.Id)){
								Floater.Update(currentalbum.Id);
								}
							ArtworkImage.reload();
							}
						else{
							UI.ShowResultInfo("No Artwork added");
							}
						}
					else{
						UI.ShowResultInfo("Add Artwork: No Album found");
						}
					}
				else{
					UI.ShowResultInfo("Add Artwork: Unknown File Format");
					}
				}
			currentalbum = null;
			}
		catch(ex){
			log("__Error adding artwork to album: " + ex);
			}
		UI.Busy = false;
		StateTimer.ticking = true;
		ArtworkTitlefield.scrolling = "autoLeft";
		fxCheckTrackDisplay();	
		}
	, DisplayPlayingAlbum: function(){
		if(this.IsRunning() && !UI.Busy && !UI.Iconized){
			if( this.IsPlaying() ){
				if( !this.Track.IsStream ){
					if(UI.SearchMode){ UI.CloseSearch(); }
					if(UI.Minimized){ UI.Minimize(); }
					var trackInfo = this.GetTrack();
					var key = fxGenerateAlbumId(trackInfo.Artist, trackInfo.Album, trackInfo.IsCompilation);
					Browser.ShowAlbumByKey(key);
					trackInfo = null;
					}
				}
			}
		}
	, DockPlayingTrack: function(){
		if( this.IsRunning() && UI.DockVitality && widget.dockOpen){
			if( this.IsPlaying() ){
				var track = iTunesObj.CurrentTrack;
				if(track != null){
					var artworkPath = "";
					var rating = null;
					switch(track.Kind){
						case 1:
							var key = fxGenerateAlbumId(track.Artist, track.Album, !!track.Compilation);
							isNewAlbum = !Albums.Exists(key);
							if(isNewAlbum){
								artworkPath = this.SaveArtworkToFile(track.Artwork, system.widgetDataFolder, "DockArtwork");
								}
							else{
								var album = Albums.GetAlbum(key);
								artworkPath = album.GetThumbnailPath();
								album = null;
								}
							rating = track.Rating;
							break;
						case 2:
							artworkPath = Const.NoArtworkImage;
							rating = null;
							break;
						case 3:
							artworkPath = Const.NoArtworkImage;
							rating = null;
							break;
						default:
							artworkPath = Const.NoArtworkImage;
							rating = null;
							break;
						}
					Widget.SetDockInfo(artworkPath, rating);
					track = null;
					}
				}
			}
		}
	, DisplayPlayingTrack: function(){
		if( this.IsRunning() && UI.MiniPopup && !UI.Busy){
			if( this.IsPlaying() ){
				var trackInfo = this.GetTrack();
				var isNewAlbum = false;
				var artworkPath = "";
				var skipAction = ( trackInfo.IsVideo || (!Library.UseCompilation && trackInfo.IsCompilation) || trackInfo.IsPodcast );
				if(!trackInfo.IsFileTrack){
					isNewAlbum = false;
					artworkPath = Const.NoArtworkImage;
					}
				else{
					var key = fxGenerateAlbumId(trackInfo.Artist, trackInfo.Album, trackInfo.IsCompilation);
					isNewAlbum = !Albums.Exists(key);
					if(isNewAlbum){
						artworkPath = this.SaveArtworkToFile(iTunesObj.CurrentTrack.Artwork, system.widgetDataFolder, "MiniPopupArtwork");
						}
					else{
						var album = Albums.GetAlbum(key);
						artworkPath = album.GetThumbnailPath();
						album = null;
						}
					}
				if(isNewAlbum && skipAction){ isNewAlbum = false; }
				UI.ShowMiniPopup(artworkPath, isNewAlbum);
				trackInfo = null;
				}
			}
		}
	, Search: function(title){
		if(title.length == 0){ return null; }
		if(title.length >= 255){ title = title.substr(0, 250); }
		return iTunesObj.LibraryPlaylist.Search(title, 3);
		}
	, GetPlaylist: function(name, autocreate){
		var	playlists = iTunesObj.LibrarySource.Playlists;
		var playlist = null;
		playlist = playlists.ItemByName(name);
		if(playlist == null && autocreate){
			playlist = iTunesObj.CreatePlaylist(name);	
			}
		playlists = null;
		return playlist;
		}
	, PlayAlbum: function(key){
		try{
			if(this.IsRunning()){
				var currentalbum = Albums.ItemByKey(key);
				var albumTracks = this.Search(currentalbum.Title);
				if(albumTracks != null){
					var playlist = this.GetPlaylist(this.PlaylistName, true);
					if(playlist != null){
						UI.ShowResultInfo("Playing Album...");
						var currenttracks = playlist.Tracks;
						for(var j = currenttracks.Count; j > 0; j--){
							currenttracks.Item(j).Delete();
							}
						currenttracks = null;
						var result = this.AddTracksToPlaylist(playlist, albumTracks, currentalbum.Artist, currentalbum.Title);
						if(result.added == 0){
							if(result.filtered > 0){
								UI.ShowResultInfo("No Tracks Added ("+ result.filtered +" filtered out)");
								}
							else{
								UI.ShowResultInfo("Play Album: No Tracks Added");
								}
							}
						else{
							iTunesObj.Stop();
							sleep(100);
							playlist.PlayFirstTrack();
							if(result.filtered > 0){
								UI.ShowResultInfo("Playing "+ result.added + ((result.added==1)?" Track":" Tracks") + " ("+ result.filtered +" filtered out)");
								}
							else{
								UI.ShowResultInfo("Playing "+ result.added + ((result.added==1)?" Track":" Tracks"));	
								}
							}
						result = null;
						}
					playlist = null;
					}
				else{
					UI.ShowResultInfo("Play Album: No Tracks Found");
					}
				albumTracks = null;
				currentalbum = null;
				}
			}
		catch(ex){
			log("__Error playing album:" + ex);
			}
		}
	, AddAlbum: function(key){
		try{
			if(this.IsRunning()){
				var currentalbum = Albums.ItemByKey(key);
				var albumTracks = this.Search(currentalbum.Title);
				if(albumTracks != null){
					var result;
					var playlist = this.GetPlaylist(this.PlaylistName, true);
					if(playlist != null){
						UI.ShowResultInfo("Adding Album to Playlist...");
						result = this.AddTracksToPlaylist(playlist, albumTracks, currentalbum.Artist, currentalbum.Title);
						}
					playlist = null;
					if(result.added == 0){
						if(result.filtered > 0){
							UI.ShowResultInfo("No Tracks Added ("+ result.filtered +" filtered out)");	
							}
						else{
							UI.ShowResultInfo("Add Album: No Tracks Added");
							}
						}
					else{
						if(result.filtered > 0){
							UI.ShowResultInfo(result.added + ((result.added==1)?" Track":" Tracks") + " Added ("+ result.filtered +" filtered out)");
							}
						else{
							UI.ShowResultInfo(result.added + ((result.added==1)?" Track":" Tracks") + " Added");
							}
						}
					result = null;
					}
				else{
					UI.ShowResultInfo("Add Album: No Tracks Found");
					}
				currentalbum = null;
				albumTracks = null;
				}
			}
		catch(ex){
			log("__Error adding album to playlist:" + ex);
			}
		}
	, AddTracksToPlaylist: function(playlist, albumTracks, artist, title){
		try{
			var addedCount = 0;
			var filteredCount = 0;
			var onlychecked = (preferences.PlaylistCheckedItems.value == 1);
			var filter = preferences.PlaylistFilter.value;
			var rating, onlyunrated;
			if(filter == "unrated"){
				onlyunrated = true;
				}
			else{
				rating = parseInt(filter);
				onlyunrated = false;
				}
			var albumCount = albumTracks.Count;
			for(var k = 1; k <= albumCount; k++){
				var track = albumTracks.Item(k);
				if(trim(track.Album) == title){
					var isVideo = fxIsVideo(track);
					if(isVideo || (!Library.UseCompilation && track.Compilation) ){
						track = null;
						continue;
						}
					if(((!track.Compilation)?(trim(track.Artist) == artist):true) && ((onlychecked)?track.Enabled:true)){
						if(( (onlyunrated)?(track.Rating == 0):(track.Rating >= rating) )){
							playlist.AddTrack(track);
							addedCount += 1;
							}
						else{
							filteredCount += 1;
							}
						}
					}
				track = null;
				}
			return { added: addedCount , filtered: filteredCount };
			}
		catch(ex){
			log("__Error adding tracks to playlist:" + ex);
			}
		}
	, AddCurrentTrackToPlaylist: function(name){
		try{
			var playlist = null;
			if(name == "Party Shuffle"){
				var	playlists = iTunesObj.LibrarySource.Playlists;
				var count = playlists.Count;
				for(var i = 1; i <= count; i++){
					var pl = playlists.Item(i);
					if(pl.Kind == 2){
						if(pl.SpecialKind == 2){
							playlist = pl;
							break;
							}
						}
					}
				}
			else{
				playlist = this.GetPlaylist(name, true);
				}
			if(playlist != null){
				var track = iTunesObj.CurrentTrack;
				playlist.AddTrack(track);
				}
			}
		catch(ex){
			log("__Error adding tracks to playlist:" + ex);
			}
		}
	, SetStateOnControls: function(){
		if(this.IsRunning()){
			RewindButton.Enable();
			PlayButton.Enable();
			ForwardButton.Enable();
			VolumeDownButton.Enable();
			VolumeUpButton.Enable();
			TrackDisplay.ChangePlayState();
			}
		else{
			RewindButton.Disable();
			PlayButton.Disable();
			ForwardButton.Disable();
			VolumeDownButton.Disable();
			VolumeUpButton.Disable();
			}
		}
	, PlayPlaylist: function(name){
		try{
			var playlist = null;
			var	playlists = iTunesObj.LibrarySource.Playlists;
			playlist = playlists.ItemByName(name);
			if(playlist!= null){
				UI.ShowResultInfo("Playing Selected Playlist...");
				iTunesObj.BrowserWindow.SelectedPlaylist = playlist;
				playlist.PlayFirstTrack();
				}
			else{
				UI.ShowResultInfo("No Playlist Found");
				this.CachePlaylists();
				}
			playlist = null;
			playlists = null;
			}
		catch(ex){
			log("__Error selecting new playlist: " + ex);
			}
		}
	, CachePlaylists: function(){
		try{
			this.PlaylistCache = null; 
			this.PlaylistCache = new Array();
			var	playlists = iTunesObj.LibrarySource.Playlists;
			var playlistCount = playlists.Count;
			for(var p = 1; p <= playlistCount; p++){
				var playlist = playlists.Item(p);
				if(playlist.Kind == 2){
					if(playlist.SpecialKind <= 2){
						this.PlaylistCache.push(playlist.Name);
						}
					}
				}
			var sortFx = function(a, b){
							if(a.toLowerCase() > b.toLowerCase()) return 1;
							if(a.toLowerCase() < b.toLowerCase()) return -1;
							return 0;
							}; 
			this.PlaylistCache.sort(sortFx);
			playlists = null;
			sortFx = null;
			}
		catch(ex){
			log("__Error caching playlists: " + ex);
			}	
		}
	, SetVolume: function(newvolume){
		this.Volume = Math.min(Math.max(0, newvolume),100);
		UI.ShowResultInfo("Volume: " + this.Volume + "%");
		}
	, SetTrackRating: function(userRating){
		try{
			if(this.IsPlaying()){
				this.Rating = userRating;
				if((userRating > 0 && userRating <= 40) && (preferences.LowRatingSkip.value == 1)){
					this.NextTrack();
					}
				}
			}
		catch(ex){
			log("__Error adding rating to current playing track: " + ex);
			}
		}
	};