var Source = {
	CreateDoc: function(){
		return XMLDOM.createDocument();
		}
	, OpenDoc: function(){
		return XMLDOM.parse( filesystem.readFile( Const.LibraryFilePath ) );
		}
	, SaveDoc: function(document){
		filesystem.writeFile( Const.LibraryFilePath, document.toXML() );
		}
	, FindNode: function(document, key){
		var node = null;
		try{
			var albums = document.documentElement.childNodes;
			var numAlbums = albums.length;
			for(var i = 0; i < numAlbums; i++){
				if(albums.item(i).getAttribute("id") == key){
					node = albums.item(i);
					break;
					}
				}
			}
		catch(ex){
			node = null;
			log("__Error finding album in source: " + ex);
			}
		return node;
		}
	, CreateNode: function(doc, album){
		var node = null;
		try{
			var childnode;
			node = doc.createElement( "album" );
			node.setAttribute("id", album.Id);
			node.setAttribute("added", album.DateAdded.valueOf() );
			childnode = doc.createElement( "artist" );
			childnode.appendChild(doc.createTextNode( album.Artist ));
			node.appendChild(childnode);
			childnode = doc.createElement( "title" );
			childnode.appendChild(doc.createTextNode( album.Title ));
			node.appendChild(childnode);
			childnode = doc.createElement( "year" );
			childnode.appendChild(doc.createTextNode( album.Year ));
			node.appendChild(childnode);
			childnode = doc.createElement( "tracks" );
			childnode.setAttribute("count", album.TrackCount);
			childnode.setAttribute("rated", album.RatedTrackCount);
			childnode.setAttribute("total", album.TotalRating);
			childnode.setAttribute("average", album.AverageRating);
			node.appendChild(childnode);
			childnode = doc.createElement( "artwork" );
			childnode.appendChild(doc.createTextNode( album.ArtworkName ));
			childnode.setAttribute("type", album.ArtworkSource);
			childnode.setAttribute("mirror", album.HasReflection);
			node.appendChild(childnode);
			}
		catch(ex){
			node = null;
			log("__Error creating album node: " + ex);
			}
		return node;
		}
	, Save: function(){
		try{
			var doc = this.CreateDoc();
			var root = doc.createElement( "albums" );
			root.setAttribute("created", Date());
			root.setAttribute("version", "2.2");
			doc.appendChild( root );
			var numAlbums = Albums.Count();
			for(var j = 0; j < numAlbums; j++){
				root.appendChild( this.CreateNode( doc, Albums.ItemByIndex(j) ) );
				}
			this.SaveDoc(doc);
			}
		catch(ex){
			log("__Error saving Library to source: " + ex);
			}
		}
	, GetVersion: function(){
		var version;
		try{
			var doc, root;
			doc = this.OpenDoc();
			var root = doc.documentElement;
			version = root.getAttribute("version");
			doc = null;
			root = null;
			}
		catch(ex){
			log("__Error retriving version from from the datafile: " + ex);
			version = "";
			}
		return version;
		}
	
	, Fill: function(){
		var doc, albums;
		var	numAlbums;
		var startTimestamp, currentTimestamp, timeleft;
		try{
			doc = this.OpenDoc();
			albums = doc.documentElement.childNodes;
			numAlbums = albums.length;
			Albums.Clear();
			startTimestamp = fxGetTimestamp();
			for(var i = 0; i < numAlbums; i++){
				var albumnode = albums.item(i);
				var datanodes = albumnode.childNodes;
				var album = new Album(albumnode.getAttribute("id"), datanodes.item(1).firstChild.data, datanodes.item(0).firstChild.data);
				album.Year = parseInt(datanodes.item(2).firstChild.data);
				album.DateAdded = new Date(parseInt(albumnode.getAttribute("added")));
				album.TrackCount = datanodes.item(3).getAttribute("count");
				album.RatedTrackCount = datanodes.item(3).getAttribute("rated");
				album.TotalRating = datanodes.item(3).getAttribute("total");
				album.AverageRating = datanodes.item(3).getAttribute("average");
				var artworkSource = parseInt( datanodes.item(4).getAttribute("type") );
				if(artworkSource > 0){
					album.HasArtwork = true;
					album.ArtworkName = datanodes.item(4).firstChild.data;
					album.ArtworkSource = artworkSource;
					var reflection = datanodes.item(4).getAttribute("mirror");
					album.HasReflection = (datanodes.item(4).getAttribute("mirror") == "true");
					}
				Albums.Add(album);
				datanodes = null
				albumnode = null;
				album = null;
				if(i > 0 && i % 20 == 0){
					currentTimestamp = fxGetTimestamp();
					timeleft = Math.round(((currentTimestamp - startTimestamp)/ i) * (numAlbums - i) + 490);
					UI.UpdateIndexFrame( Math.round((i / (numAlbums - 1)) * 100), timeleft );
					}
				}
			doc = null;
			albums = null;
			}
		catch(ex){
			log("__Error building Library from source: " + ex);
			}
		}
	, Build: function(){
		try{
			var startTimestamp, timeleft;
			var	mainLibrary = iTunesObj.LibraryPlaylist;
			var	tracks = mainLibrary.Tracks;
			var numTracks = tracks.Count;
			Albums.Clear();
			startTimestamp = fxGetTimestamp();
			for(var i = 1; i <= numTracks; i++){
				var	track = tracks.Item(i);
				var isCompilation = track.Compilation;
				var isPodcast = track.Podcast;
				var isVideo = fxIsVideo(track);
				if( isVideo || (!Library.UseCompilation && isCompilation) || isPodcast ){
					track = null;
					continue;
					}
				var	title = trim(track.Album);
				var artist = trim(track.Artist);
				if(title != "" && artist != ""){
					var key = fxGenerateAlbumId(artist, title, isCompilation);
					var index = Albums.IndexOf(key);
					if(index == null){
						var album = new Album(key, title, (!isCompilation)?artist:Const.CompilationAlbum);
						album.AddTrack(track);
						Albums.Add(album);
						album = null;
						}
					else{
						Albums.ItemByIndex(index).AddTrack(track);
						}
					}
				track = null;
				if(i % 16 == 0){
					timeleft = ((fxGetTimestamp() - startTimestamp)/ i) * (numTracks - i) + 490;
					UI.UpdateIndexFrame( Math.round((i / numTracks) * 100), Math.round(timeleft) );
					}
				}
			var runtime = fxGetTimeLeftString(fxGetTimestamp() - startTimestamp);
			this.Save();
			}
		catch(ex){
			log("__Error building Library from iTunes data: " + ex);
			}
		}
	, Add: function(album){
		var result = false;
		try{
			var key = album.Id;
			var doc = this.OpenDoc();
			var albumnode = this.CreateNode(doc, album);
			if(albumnode != null){
				var root = doc.documentElement;
				root.appendChild(albumnode);
				this.SaveDoc(doc);
				result = true;
				root = null;
				}
			albumnode = null;
			doc = null;
			}
		catch(ex){
			log("__Error adding album to data source: " + ex);
			}
		return result;
		}
	, Update: function(key, album){
		var result = false;
		try{
			var doc = this.OpenDoc();
			var oldnode = this.FindNode(doc, key);
			if(oldnode != null){
				var albumnode = this.CreateNode(doc, album);
				if(albumnode != null){
					var root = doc.documentElement;
					root.replaceChild(albumnode, oldnode);
					this.SaveDoc(doc);
					result = true;
					root = null;
					}
				albumnode = null;
				}
			doc = null;
			oldnode = null;
			}
		catch(ex){
			log("__Error updating album in data source: " + ex);
			}
		return result;
		}
	, Remove: function(key){
		var result = false;
		try{
			var doc = this.OpenDoc();
			var oldnode = this.FindNode(doc, key);
			if(oldnode != null){	
				var root = doc.documentElement;
				root.removeChild(oldnode);
				this.SaveDoc(doc);
				result = true;
				root = null;
				}
			doc = null;
			oldnode = null;
			}
		catch(ex){
			log("__Error removing album from data source: " + ex);
			}
		return result;
		}
	};